package com.elitesland.core.base;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.annotations.GenericGenerator;
import org.springframework.data.annotation.CreatedBy;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.LastModifiedBy;
import org.springframework.data.annotation.LastModifiedDate;
import org.springframework.data.jpa.domain.support.AuditingEntityListener;

import javax.persistence.*;
import javax.validation.constraints.Null;
import java.time.LocalDateTime;

/**
 * <pre>
 *     JPA数据实体类的基础类，用于唯一ID生成策略绑定
 *     审计字段、版本和逻辑删除标记，
 *     未来扩展租户ID信息
 * </pre>
 * @author Moz
 * @date 3/17/2020
 */
@Data
@EntityListeners(AuditingEntityListener.class)
@MappedSuperclass
public class BaseModel {
    @Id
    @GenericGenerator(name = "el-id", strategy = "com.elitesland.core.util.IdGenerator")
    @GeneratedValue(strategy = GenerationType.AUTO, generator = "el-id")
    @JsonSerialize(using = ToStringSerializer.class)
    @ApiModelProperty("账号唯一ID")
    @Column(columnDefinition = "bigint default 0 comment '唯一编号ID'")
    private Long id;

    @CreatedBy
    @Column(columnDefinition = "bigint default 0 comment '记录创建者ID'")
    private Long creator;

    @CreatedDate
    @Column(columnDefinition = "timestamp default NULL comment '记录创建时间'")
    private LocalDateTime created = LocalDateTime.now();

    @LastModifiedBy
    @Column(columnDefinition = "bigint default 0 comment '记录最后更新者ID'")
    private Long updater;

    @LastModifiedDate
    @Column(columnDefinition = "timestamp default NULL comment '记录最后更新时间'")
    private LocalDateTime updated = LocalDateTime.now();

    @ApiModelProperty(value = "逻辑删除，0：未删除，1：已删除")
    @Null(message = "逻辑删除不用传")
    @Column(columnDefinition = "int default 0 comment '逻辑删除，0：未删除，1：已删除'")
    private Integer deleted;

    @ApiModelProperty(value = "版本")
    @Null(message = "版本不用传")
    @Column(columnDefinition = "int default 0 comment '版本信息，前端不用传'")
    private Integer version;
}
