package com.elitesland.core.base.param;

import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.querydsl.core.types.Order;
import com.querydsl.core.types.OrderSpecifier;
import com.querydsl.core.types.Predicate;
import com.querydsl.core.types.dsl.PathBuilder;
import com.querydsl.jpa.impl.JPAQuery;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.val;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.repository.query.JpaQueryCreator;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 可排序查询参数对象
 *
 * @author Mir
 * @date 2019-08-04
 */
@Data
@EqualsAndHashCode(callSuper = true)
@ApiModel("可排序查询参数对象")
public abstract class AbstractOrderQueryParam extends QueryParam {
    private static final long serialVersionUID = 57714391204790143L;
    @ApiModelProperty(value = "排序")
    private List<OrderItem> orders;

    public List<OrderItem> getOrders() {
        return orders;
    }

    public void setOrders(List<OrderItem> orders) {
        this.orders = orders;
    }

    public void defaultOrder(OrderItem orderItem) {
        this.defaultOrders(Collections.singletonList(orderItem));
    }

    public void defaultOrders(List<OrderItem> orderItems) {
        if (CollectionUtils.isEmpty(orderItems)) {
            return;
        }
        this.orders = orderItems;
    }

    @JsonIgnore
    public PageRequest getPageRequest() {
        val orderBys = CollectionUtils.isEmpty(orders) ? new ArrayList<Sort.Order>() :
                orders.stream().map(o -> new Sort.Order(
                        o.isAsc() ? Sort.Direction.ASC : Sort.Direction.DESC,
                        o.getColumn()
                )).collect(Collectors.toList());

        return PageRequest.of(getCurrent(), getSize(), Sort.by(orderBys));
    }

    @JsonIgnore
    public void fillOrders(JPAQuery<?> query){
        val pageRequest = getPageRequest();
        pageRequest.getSort().forEach(s -> {
            val orderbyExpression = new PathBuilder<>(Object.class, "object");
            query.orderBy(new OrderSpecifier(s.isAscending() ?
                    Order.ASC : Order.DESC, orderbyExpression.get(s.getProperty())));
        });
    }

    @JsonIgnore
    public void setPaging(JPAQuery<?> query){
        query.offset(getCurrent() * getSize());
        query.limit(getSize());
    }
}
